@@ -142,6 +142,7 @@ typedef enum {
     S2N_ERR_MAX_EARLY_DATA_SIZE,
     S2N_ERR_EARLY_DATA_TRIAL_DECRYPT,
     S2N_ERR_NO_RENEGOTIATION,
+    S2N_ERR_KTLS_KEYUPDATE,
     S2N_ERR_T_PROTO_END,
 
     /* S2N_ERR_T_INTERNAL */
@@ -319,6 +320,7 @@ typedef enum {
     S2N_ERR_KTLS_BAD_CMSG,
     S2N_ERR_KTLS_RENEG,
     S2N_ERR_ATOMIC,
+    S2N_ERR_KTLS_KEY_LIMIT,
     S2N_ERR_T_USAGE_END,
 } s2n_error;
 
