@@ -472,21 +472,6 @@ class OpenSSL(Provider):
 
     @classmethod
     def supports_cipher(cls, cipher, with_curve=None):
-        if "openssl-1.0.2" in get_flag(S2N_PROVIDER_VERSION) and with_curve is not None:
-            invalid_ciphers = [
-                Ciphers.ECDHE_RSA_AES128_SHA,
-                Ciphers.ECDHE_RSA_AES256_SHA,
-                Ciphers.ECDHE_RSA_AES128_SHA256,
-                Ciphers.ECDHE_RSA_AES256_SHA384,
-                Ciphers.ECDHE_RSA_AES128_GCM_SHA256,
-                Ciphers.ECDHE_RSA_AES256_GCM_SHA384,
-            ]
-
-            # OpenSSL 1.0.2 and 1.0.2-FIPS can't find a shared cipher with S2N
-            # when P-384 is used, but I can't find any reason why.
-            if with_curve is Curves.P384 and cipher in invalid_ciphers:
-                return False
-
         return True
 
     def _is_openssl_11(self) -> None:
