@@ -70,7 +70,7 @@ struct s2n_security_policy {
      * https://www.rfc-editor.org/rfc/rfc8446#section-4.2.7
      */
     const struct s2n_ecc_preferences *ecc_preferences;
-    s2n_security_flag rules;
+    bool rules[S2N_SECURITY_RULES_COUNT];
 };
 
 struct s2n_security_policy_selection {
